@@ -1,35 +1,34 @@
 # -*- coding: UTF-8 -*-
-# Name: filter_crop.py
-# Porpose: Show dialog to get video crop values based on FFmpeg syntax
-# Compatibility: Python3, wxPython Phoenix
-# Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
-# Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
-# license: GPL3
-# Rev: Dec.14.2020 *PEP8 compatible*
-#########################################################
+"""
+Name: filter_crop.py
+Porpose: Show dialog to get video crop values based on FFmpeg syntax
+Compatibility: Python3, wxPython Phoenix
+Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
+Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
+license: GPL3
+Rev: May.09.2021 *-pycodestyle- compatible*
+########################################################
 
-# This file is part of Videomass.
+This file is part of Videomass.
 
-#    Videomass is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation, either version 3 of the License, or
-#    (at your option) any later version.
+   Videomass is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
-#    Videomass is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+   Videomass is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-#    You should have received a copy of the GNU General Public License
-#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
-
-#########################################################
-import wx
-import webbrowser
-# import wx.lib.masked as masked  # not work on macOSX
-import wx.lib.statbmp
+   You should have received a copy of the GNU General Public License
+   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
+"""
 import os
 from time import sleep
+import wx
+# import wx.lib.masked as masked  # not work on macOSX
+import wx.lib.statbmp
 from videomass3.vdms_threads.generic_task import FFmpegGenericTask
 from videomass3.vdms_utils.utils import get_milliseconds
 from videomass3.vdms_utils.utils import milliseconds2timeformat
@@ -94,27 +93,28 @@ class Actor(wx.lib.statbmp.GenStaticBitmap):
         dc.DrawBitmap(self.current_bmp, 0, 0, True)
         dc.SetPen(wx.Pen('red', 2, wx.PENSTYLE_SOLID))
         dc.SetBrush(wx.Brush('green', wx.BRUSHSTYLE_TRANSPARENT))
-        dc.DrawRectangle(self.x, self.y, self.w, self.h)
+        dc.DrawRectangle(self.x + 1, self.y + 1, self.w + 2, self.h + 2)
     # ------------------------------------------------------------------#
 
     def onRedraw(self, x, y, w, h):
         """
-        Update Drawing: A transparent background rectangle in a bitmap
-        object. To compensate for the PEN thickness offset, the positions
-        are increased by 1 and the sizes decreased by 1 .
+        Update Drawing: A transparent background rectangle in a
+        bitmap object. To compensate for the PEN thickness offset,
+        the sizes are increased by 2 and the positions decreased
+        by 1 (even on OnPaint)
 
         NOTE dc.SetBrush(wx.Brush(wx.Colour(30, 30, 30, 128))) would set
         a useful transparent gradation color but it doesn't work on windows
         and gtk2.
 
         """
-        self.h, self.w, self.x, self.y = h - 1, w - 1, x + 1, y + 1
+        self.h, self.w, self.x, self.y = h, w, x, y
         dc = wx.ClientDC(self)
         dc.Clear()  # needed if image has trasparences
         dc.DrawBitmap(self.current_bmp, 0, 0, True)
         dc.SetPen(wx.Pen('red', 2, wx.PENSTYLE_SOLID))
         dc.SetBrush(wx.Brush('green', wx.BRUSHSTYLE_TRANSPARENT))
-        dc.DrawRectangle(self.x, self.y, self.w, self.h)
+        dc.DrawRectangle(self.x + 1, self.y + 1, self.w + 2, self.h + 2)
 
 
 class Crop(wx.Dialog):
@@ -123,8 +123,9 @@ class Crop(wx.Dialog):
 
     """
     get = wx.GetApp()
-    OS = get.OS
-    TMP = get.TMP
+    OS = get.appset['ostype']
+    TMP = os.path.join(get.appset['cachedir'], 'tmp')
+    DISPLAY_SIZE = get.appset['DISPLAY_SIZE']
     BACKGROUND = '#1b0413'
     # ------------------------------------------------------------------#
 
@@ -158,7 +159,7 @@ class Crop(wx.Dialog):
         self.v_width = v_width
         self.v_height = v_height
         # resizing values preserving aspect ratio for monitor
-        self.thr = 180 if self.v_height > self.v_width else 360
+        self.thr = 180 if self.v_height >= self.v_width else 270
         self.h_ratio = (self.v_height / self.v_width) * self.thr  # height
         self.w_ratio = (self.v_width / self.v_height) * self.h_ratio  # width
 
@@ -172,7 +173,7 @@ class Crop(wx.Dialog):
             self.image = wx.Bitmap(self.w_ratio, self.h_ratio)  # make empty
 
         duration = get_milliseconds(timeformat)  # convert to ms
-        hhmmss = milliseconds2timeformat(duration)  # convert to timeformat
+        # hhmmss = milliseconds2timeformat(duration)  # convert to timeformat
 
         wx.Dialog.__init__(self, parent, -1, style=wx.DEFAULT_DIALOG_STYLE)
         sizerBase = wx.BoxSizer(wx.VERTICAL)
@@ -181,112 +182,89 @@ class Crop(wx.Dialog):
                                   )
         bmp = make_bitmap(self.w_ratio, self.h_ratio, self.image)
         self.bob = Actor(self.panelrect, bmp, 1, "")
-        sizerBase.Add(self.panelrect, 0, wx.TOP | wx.CENTER, 10)
+        sizerBase.Add(self.panelrect, 0, wx.ALL | wx.CENTER, 5)
         sizersize = wx.BoxSizer(wx.VERTICAL)
-        sizerBase.Add(sizersize, 0, wx.ALL | wx.CENTER, 10)
+        sizerBase.Add(sizersize, 0, wx.ALL | wx.CENTER, 5)
         msg = _("Source size: {0} x {1} pixels").format(self.v_width,
                                                         self.v_height)
         label1 = wx.StaticText(self, wx.ID_ANY, msg)
-        sizersize.Add(label1, 0, wx.CENTER, 10)
-        msg = _("Search for a specific frame, max "
-                "duration {}").format(hhmmss)
+        sizersize.Add(label1, 0, wx.CENTER)
+        msg = _("Search for a specific frame")
         sizer_load = wx.StaticBoxSizer(wx.StaticBox(self, wx.ID_ANY, (msg)),
                                        wx.HORIZONTAL)
-        sizerBase.Add(sizer_load, 0, wx.ALL | wx.CENTRE, 10)
+        sizerBase.Add(sizer_load, 0, wx.ALL | wx.EXPAND, 5)
 
         self.txttime = wx.StaticText(self, wx.ID_ANY, '00:00:00.000')
         sizer_load.Add(self.txttime, 0, wx.ALL | wx.CENTER, 10)
         self.slider = wx.Slider(self, wx.ID_ANY, 0, 0, duration,
-                                size=(300, -1), style=wx.SL_HORIZONTAL
+                                size=(150, -1), style=wx.SL_HORIZONTAL
                                 )
         sizer_load.Add(self.slider, 0, wx.ALL | wx.CENTER, 5)
         btn_load = wx.Button(self, wx.ID_ANY, _("Load"))
-        sizer_load.Add(btn_load, 0, wx.ALL | wx.CENTER, 10)
+        sizer_load.Add(btn_load, 1, wx.ALL | wx.EXPAND, 10)
         sizerLabel = wx.StaticBoxSizer(wx.StaticBox(self, wx.ID_ANY, (
-                        _("Area selection and position setting in pixels"))),
+                        _("Cropping area selection "))),
                                        wx.VERTICAL)
-        sizerBase.Add(sizerLabel, 1, wx.ALL | wx.EXPAND, 10)
+        sizerBase.Add(sizerLabel, 0, wx.ALL | wx.EXPAND, 5)
         boxctrl = wx.BoxSizer(wx.VERTICAL)
-        sizerLabel.Add(boxctrl, 0, wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 15)
+        sizerLabel.Add(boxctrl, 0, wx.CENTRE)
         label_height = wx.StaticText(self, wx.ID_ANY, (_("Height")))
-        boxctrl.Add(label_height, 0, wx.ALL |
-                    wx.ALIGN_CENTER_HORIZONTAL, 5
-                    )
+        boxctrl.Add(label_height, 0, wx.ALL | wx.CENTRE, 0)
         self.crop_height = wx.SpinCtrl(self, wx.ID_ANY, "0",
                                        min=0, max=self.v_height, size=(-1, -1),
                                        style=wx.TE_PROCESS_ENTER |
                                        wx.SP_ARROW_KEYS
                                        )
-        boxctrl.Add(self.crop_height, 0, wx.ALL |
-                    wx.ALIGN_CENTER_HORIZONTAL, 5
-                    )
+        boxctrl.Add(self.crop_height, 0, wx.CENTRE)
         grid_sizerBase = wx.FlexGridSizer(1, 5, 0, 0)
-        boxctrl.Add(grid_sizerBase, 1, wx.EXPAND, 0)
+        boxctrl.Add(grid_sizerBase, 0, wx.CENTRE, 0)
         label_X = wx.StaticText(self, wx.ID_ANY, ("X"))
-        grid_sizerBase.Add(label_X, 0, wx.ALL |
-                           wx.ALIGN_CENTER_HORIZONTAL |
-                           wx.ALIGN_CENTER_VERTICAL, 5
-                           )
+        grid_sizerBase.Add(label_X, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
+
         self.axis_X = wx.SpinCtrl(self, wx.ID_ANY, "0",
                                   min=-1, max=self.v_width, size=(-1, -1),
                                   style=wx.TE_PROCESS_ENTER |
                                   wx.SP_ARROW_KEYS
                                   )
-        grid_sizerBase.Add(self.axis_X, 0, wx.ALL |
-                           wx.ALIGN_CENTER_HORIZONTAL |
-                           wx.ALIGN_CENTER_VERTICAL, 5
-                           )
+        grid_sizerBase.Add(self.axis_X, 0, wx.ALL | wx.CENTRE, 5)
+
         self.btn_centre = wx.Button(self, wx.ID_ANY, _("Center"))
-        grid_sizerBase.Add(self.btn_centre, 0, wx.ALL | wx.CENTRE, 35
-                           )
+        grid_sizerBase.Add(self.btn_centre, 0, wx.ALL |
+                           wx.ALIGN_CENTER_VERTICAL, 5)
+
         self.crop_width = wx.SpinCtrl(self, wx.ID_ANY, "0",
                                       min=0,  max=self.v_width, size=(-1, -1),
                                       style=wx.TE_PROCESS_ENTER |
                                       wx.SP_ARROW_KEYS
                                       )
-        grid_sizerBase.Add(self.crop_width, 0, wx.ALL |
-                           wx.ALIGN_CENTER_HORIZONTAL |
-                           wx.ALIGN_CENTER_VERTICAL, 5
-                           )
+        grid_sizerBase.Add(self.crop_width, 0, wx.ALL | wx.CENTRE, 5)
+
         label_width = wx.StaticText(self, wx.ID_ANY, (_("Width")))
-        grid_sizerBase.Add(label_width, 0, wx.ALL |
-                           wx.ALIGN_CENTER_HORIZONTAL |
-                           wx.ALIGN_CENTER_VERTICAL, 5
-                           )
+        grid_sizerBase.Add(label_width, 0, wx.RIGHT |
+                           wx.ALIGN_CENTER_VERTICAL, 5)
         self.axis_Y = wx.SpinCtrl(self, wx.ID_ANY, "0",
                                   min=-1, max=self.v_height, size=(-1, -1),
                                   style=wx.TE_PROCESS_ENTER |
                                   wx.SP_ARROW_KEYS
                                   )
-        boxctrl.Add(self.axis_Y, 0, wx.ALL |
-                    wx.ALIGN_CENTER_HORIZONTAL |
-                    wx.ALIGN_CENTER_VERTICAL, 5
-                    )
+        boxctrl.Add(self.axis_Y, 0, wx.CENTRE)
         label_Y = wx.StaticText(self, wx.ID_ANY, ("Y"))
-        boxctrl.Add(label_Y, 0, wx.ALL |
-                    wx.ALIGN_CENTER_HORIZONTAL |
-                    wx.ALIGN_CENTER_VERTICAL, 5
-                    )
-        # bottom layout
+        boxctrl.Add(label_Y, 0, wx.BOTTOM | wx.CENTRE, 5)
+        # bottom layout for buttons
         gridBtn = wx.GridSizer(1, 2, 0, 0)
-        gridhelp = wx.GridSizer(1, 1, 0, 0)
-        gridBtn.Add(gridhelp)
         gridexit = wx.BoxSizer(wx.HORIZONTAL)
-        btn_help = wx.Button(self, wx.ID_HELP, "")
-        gridhelp.Add(btn_help, 0, wx.ALL, 5)
+        btn_reset = wx.Button(self, wx.ID_CLEAR, _("Reset"))
+        gridBtn.Add(btn_reset, 0, wx.ALL, 5)
         btn_close = wx.Button(self, wx.ID_CANCEL, "")
         gridexit.Add(btn_close, 0, wx.ALL, 5)
         self.btn_ok = wx.Button(self, wx.ID_OK, _("Apply"))
         gridexit.Add(self.btn_ok, 0, wx.ALL, 5)
-        btn_reset = wx.Button(self, wx.ID_CLEAR, _("Reset"))
-        gridexit.Add(btn_reset, 0, wx.ALL, 5)
         gridBtn.Add(gridexit, 0, wx.ALL | wx.ALIGN_RIGHT | wx.RIGHT, 0)
+        sizerBase.Add(gridBtn, 0, wx.EXPAND)
 
-        sizerBase.Add(gridBtn, 0, wx.ALL | wx.EXPAND, 5)
         self.SetSizer(sizerBase)
         sizerBase.Fit(self)
         self.Layout()
-
         # ----------------------Properties-----------------------#
         self.panelrect.SetBackgroundColour(wx.Colour(Crop.BACKGROUND))
         if Crop.OS == 'Darwin':
@@ -314,7 +292,7 @@ class Crop(wx.Dialog):
         self.Bind(wx.EVT_BUTTON, self.on_close, btn_close)
         self.Bind(wx.EVT_BUTTON, self.on_ok, self.btn_ok)
         self.Bind(wx.EVT_BUTTON, self.on_reset, btn_reset)
-        self.Bind(wx.EVT_BUTTON, self.on_help, btn_help)
+        # self.Bind(wx.EVT_BUTTON, self.on_help, btn_help)
 
         if timeformat == '00:00:00.000':
             self.slider.Disable(), btn_load.Disable()
@@ -492,7 +470,7 @@ class Crop(wx.Dialog):
             self.onDrawing()
 
     # ------------------------------------------------------------------#
-
+    '''
     def on_help(self, event):
         """
         Open default browser to official help page
@@ -500,6 +478,7 @@ class Crop(wx.Dialog):
         page = ('https://jeanslack.github.io/Videomass/Pages/Main_Toolbar/'
                 'VideoConv_Panel/Filters/FilterCrop.html')
         webbrowser.open(page)
+    '''
     # ------------------------------------------------------------------#
 
     def on_reset(self, event):
@@ -519,7 +498,8 @@ class Crop(wx.Dialog):
     # ------------------------------------------------------------------#
 
     def on_ok(self, event):
-        """
+        """22077??VvA
+
         if you enable self.Destroy(), it delete from memory all data
         event and no return correctly. It has the right behavior if not
         used here, because it is called in the main frame.
