@@ -1,32 +1,34 @@
 # -*- coding: UTF-8 -*-
-# Name: filedrop.py
-# Porpose: files drag n drop interface
-# Compatibility: Python3, wxPython Phoenix
-# Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
-# Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
-# license: GPL3
-# Rev: Dec.04.2020 *PEP8 compatible*
-#########################################################
+"""
+Name: filedrop.py
+Porpose: files drag n drop interface
+Compatibility: Python3, wxPython Phoenix
+Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
+Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
+license: GPL3
+Rev: Aug.19.2021 *-pycodestyle- compatible*
+########################################################
 
-# This file is part of Videomass.
+This file is part of Videomass.
 
-#    Videomass is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation, either version 3 of the License, or
-#    (at your option) any later version.
+   Videomass is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
-#    Videomass is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+   Videomass is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-#    You should have received a copy of the GNU General Public License
-#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
-
-#########################################################
-import wx
+   You should have received a copy of the GNU General Public License
+   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
+"""
 import os
-from videomass3.vdms_io import IO_tools
+import sys
+import wx
+from videomass3.vdms_utils.get_bmpfromsvg import get_bmp
+from videomass3.vdms_io import io_tools
 from videomass3.vdms_utils.utils import get_milliseconds
 
 
@@ -37,9 +39,11 @@ class MyListCtrl(wx.ListCtrl):
     """
     AZURE = '#d9ffff'  # rgb form (wx.Colour(217,255,255))
     RED = '#ea312d'
-    YELLOW = '#a29500'
+    YELLOW = '#bd9f00'
     GREENOLIVE = '#6aaf23'
     ORANGE = '#f28924'
+    WHITE = '#fbf4f4'  # white for background status bar
+    BLACK = '#060505'  # black for background status bar
     # ----------------------------------------------------------------------
 
     def __init__(self, parent):
@@ -63,18 +67,29 @@ class MyListCtrl(wx.ListCtrl):
         msg_noext = _("File without format extension: please give an "
                       "appropriate extension to the file name, example "
                       "'.mkv', '.avi', '.mp3', etc.")
-        if os.path.isdir(path):
-            self.parent.statusbar_msg(msg_dir, MyListCtrl.ORANGE)
+        msg_badfn = _("Invalid filename. Contains double or single quotes")
+
+        if '"' in path or "'" in path:
+            self.parent.statusbar_msg(msg_badfn, MyListCtrl.ORANGE,
+                                      MyListCtrl.WHITE)
             return
-        elif os.path.splitext(os.path.basename(path))[1] == '':
-            self.parent.statusbar_msg(msg_noext, MyListCtrl.ORANGE)
+
+        if os.path.isdir(path):
+            self.parent.statusbar_msg(msg_dir, MyListCtrl.ORANGE,
+                                      MyListCtrl.WHITE)
+            return
+
+        if os.path.splitext(os.path.basename(path))[1] == '':
+            self.parent.statusbar_msg(msg_noext, MyListCtrl.ORANGE,
+                                      MyListCtrl.WHITE)
             return
 
         if not [x for x in self.data if x['format']['filename'] == path]:
-            data = IO_tools.probeInfo(path)
+            data = io_tools.probe_getinfo(path)
 
             if data[1]:
-                self.parent.statusbar_msg(data[1], MyListCtrl.RED)
+                self.parent.statusbar_msg(data[1], MyListCtrl.RED,
+                                          MyListCtrl.WHITE)
                 return
 
             data = eval(data[0])
@@ -106,7 +121,8 @@ class MyListCtrl(wx.ListCtrl):
 
         else:
             mess = _("Duplicate files are rejected: > '%s'") % path
-            self.parent.statusbar_msg(mess, MyListCtrl.YELLOW)
+            self.parent.statusbar_msg(mess, MyListCtrl.YELLOW,
+                                      MyListCtrl.BLACK)
     # ----------------------------------------------------------------------#
 
 
@@ -141,17 +157,22 @@ class FileDnD(wx.Panel):
     Panel for dragNdrop files queue. Accept one or more files.
     """
     # CONSTANTS:
-    get = wx.GetApp()
-    OUTSAVE = get.FFMPEGoutdir  # files destination folder
-    OS = get.OS
-    SAMEDIR = get.SAMEdir
-    SUFFIX = get.FILEsuffix
 
-    def __init__(self, parent):
+    YELLOW = '#bd9f00'
+    WHITE = '#fbf4f4'  # white for background status bar
+    BLACK = '#060505'  # black for background status bar
+
+    def __init__(self, parent, iconplay):
         """Constructor. This will initiate with an id and a title"""
+        get = wx.GetApp()
+        self.appdata = get.appset
         self.parent = parent  # parent is the MainFrame
+        if 'wx.svg' in sys.modules:  # available only in wx version 4.1 to up
+            bmpplay = get_bmp(iconplay, ((16, 16)))
+        else:
+            bmpplay = wx.Bitmap(iconplay, wx.BITMAP_TYPE_ANY)
         self.data = self.parent.data_files  # set items list data on parent
-        self.file_dest = FileDnD.OUTSAVE
+        self.file_dest = self.appdata['outputfile']
         self.selected = None  # tells if an imported file is selected or not
 
         wx.Panel.__init__(self, parent=parent)
@@ -163,60 +184,69 @@ class FileDnD(wx.Panel):
         self.flCtrl.SetDropTarget(file_drop_target)  # Make drop target.
         # create widgets
         sizer = wx.BoxSizer(wx.VERTICAL)
+        sizer.Add((0, 10))
         infomsg = _("Drag one or more files below")
-        lbl_info = wx.StaticText(self, label=infomsg)
-        sizer.Add(lbl_info, 0, wx.ALL, 5)
-        sizer.Add(self.flCtrl, 1, wx.EXPAND | wx.ALL, 5)
-        optionsmsg = _("Options")
-        lbl_options = wx.StaticText(self, label=optionsmsg)
-        sizer.Add(lbl_options, 0, wx.ALL, 5)
+        lbl_info = wx.StaticText(self, wx.ID_ANY, label=infomsg)
+        sizer.Add(lbl_info, 0, wx.ALL | wx.EXPAND, 5)
+        sizer.Add(self.flCtrl, 1, wx.EXPAND | wx.ALL, 2)
+        sizer.Add((0, 10))
         sizer_media = wx.BoxSizer(wx.HORIZONTAL)
-        sizer.Add(sizer_media, 0, wx.EXPAND | wx.ALL, 5)
-        btn_delsel = wx.Button(self, wx.ID_REMOVE, "")
-        sizer_media.Add(btn_delsel, 1, wx.ALL | wx.EXPAND, 5)
-        btn_clear = wx.Button(self, wx.ID_CLEAR, "")
-        sizer_media.Add(btn_clear, 1, wx.ALL | wx.EXPAND, 5)
-        outdirmsg = _("Output Directory")
-        lbl_outdir = wx.StaticText(self, label=outdirmsg)
-        sizer.Add(lbl_outdir, 0, wx.ALL, 5)
+        self.btn_play = wx.Button(self, wx.ID_ANY, _("Play"))
+        self.btn_play.SetBitmap(bmpplay, wx.LEFT)
+        self.btn_play.Disable()
+        sizer_media.Add(self.btn_play, 1, wx.ALL | wx.EXPAND, 2)
+        self.btn_remove = wx.Button(self, wx.ID_REMOVE, "")
+        self.btn_remove.Disable()
+        sizer_media.Add(self.btn_remove, 1, wx.ALL | wx.EXPAND, 2)
+        self.btn_clear = wx.Button(self, wx.ID_CLEAR, "")
+        self.btn_clear.Disable()
+        sizer_media.Add(self.btn_clear, 1, wx.ALL | wx.EXPAND, 2)
+        sizer.Add(sizer_media, 0, wx.EXPAND)
         sizer_outdir = wx.BoxSizer(wx.HORIZONTAL)
-        sizer.Add(sizer_outdir, 0, wx.ALL | wx.EXPAND, 5)
-        self.btn_save = wx.Button(self, wx.ID_OPEN, "...", size=(-1, -1))
-        sizer_outdir.Add(self.btn_save, 0, wx.ALL |
-                         wx.ALIGN_CENTER_HORIZONTAL |
-                         wx.ALIGN_CENTER_VERTICAL, 5
-                         )
+        self.btn_save = wx.Button(self, wx.ID_OPEN, "...", size=(35, -1))
         self.text_path_save = wx.TextCtrl(self, wx.ID_ANY, "",
                                           style=wx.TE_PROCESS_ENTER |
                                           wx.TE_READONLY
                                           )
-        sizer_outdir.Add(self.text_path_save, 1, wx.ALL | wx.EXPAND, 5)
+        sizer_outdir.Add(self.text_path_save, 1, wx.ALL | wx.EXPAND, 2)
+        sizer_outdir.Add(self.btn_save, 0, wx.ALL |
+                         wx.ALIGN_CENTER_HORIZONTAL |
+                         wx.ALIGN_CENTER_VERTICAL, 2
+                         )
+        sizer.Add(sizer_outdir, 0, wx.EXPAND)
         self.SetSizer(sizer)
         # properties
-        self.flCtrl.InsertColumn(0, _('File Name'), width=550)
+        self.flCtrl.InsertColumn(0, _('File Name'), width=350)
         self.flCtrl.InsertColumn(1, _('Duration'), width=230)
         self.flCtrl.InsertColumn(2, _('Media type'), width=200)
         self.flCtrl.InsertColumn(3, _('Size'), width=150)
-        if self.OS != 'Darwin':
-            lbl_info.SetLabelMarkup("<b>%s</b>" % infomsg)
-            lbl_options.SetLabelMarkup("<b>%s</b>" % optionsmsg)
-            lbl_outdir.SetLabelMarkup("<b>%s</b>" % outdirmsg)
+
+        if self.appdata['ostype'] == 'Darwin':
+            lbl_info.SetFont(wx.Font(12, wx.DEFAULT, wx.NORMAL, wx.BOLD))
+        else:
+            lbl_info.SetFont(wx.Font(9, wx.DEFAULT, wx.NORMAL, wx.BOLD))
+
         self.text_path_save.SetValue(self.file_dest)
 
-        if FileDnD.SAMEDIR == 'true':
+        if self.appdata['outputfile_samedir'] == 'true':
             self.btn_save.Disable(), self.text_path_save.Disable()
             self.parent.same_destin = True
-            if not FileDnD.SUFFIX == 'none':  # otherwise must be '' on parent
-                self.parent.suffix = FileDnD.SUFFIX
+            if not self.appdata['filesuffix'] == 'none':
+                # otherwise must be '' on parent
+                self.parent.suffix = self.appdata['filesuffix']
 
         # Tooltip
-        btn_delsel.SetToolTip(_('Remove the selected file from the list'))
-        btn_clear.SetToolTip(_('Delete all files from the list'))
-        tip = (_('Choose a temporary destination for conversions'))
+        self.btn_remove.SetToolTip(_('Remove the selected file from the list'))
+        self.btn_clear.SetToolTip(_('Delete all files from the list'))
+        tip = (_("Set up a temporary folder for conversions"))
         self.btn_save.SetToolTip(tip)
+        self.btn_play.SetToolTip(_("Play the selected files in the list"))
+        self.text_path_save.SetToolTip(_("Destination folder"))
+
         # Binding (EVT)
-        self.Bind(wx.EVT_BUTTON, self.deleteAll, btn_clear)
-        self.Bind(wx.EVT_BUTTON, self.delSelect, btn_delsel)
+        self.Bind(wx.EVT_BUTTON, self.playSelect, self.btn_play)
+        self.Bind(wx.EVT_BUTTON, self.deleteAll, self.btn_clear)
+        self.Bind(wx.EVT_BUTTON, self.delSelect, self.btn_remove)
         self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_select, self.flCtrl)
         self.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.on_deselect, self.flCtrl)
         self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.on_doubleClick, self.flCtrl)
@@ -231,6 +261,8 @@ class FileDnD(wx.Panel):
 
         """
         self.parent.reset_Timeline()
+        if not self.btn_clear.IsEnabled():
+            self.btn_clear.Enable()
     # ----------------------------------------------------------------------
 
     def which(self):
@@ -270,20 +302,21 @@ class FileDnD(wx.Panel):
         menuItem = menu.FindItemById(itemId)
 
         if menuItem.GetItemLabel() == _("Play"):
-            self.playSelect()
+            self.playSelect(self)
 
         elif menuItem.GetItemLabel() == _("Remove"):
             self.delSelect(self)
 
     # ----------------------------------------------------------------------
 
-    def playSelect(self):
+    def playSelect(self, event):
         """
         Playback the selected file
 
         """
         if not self.selected:
-            self.parent.statusbar_msg(_('No file selected'), 'GOLDENROD')
+            self.parent.statusbar_msg(_('No file selected'), FileDnD.YELLOW,
+                                      FileDnD.BLACK)
         else:
             self.parent.statusbar_msg(_('Add Files'), None)
             index = self.flCtrl.GetFocusedItem()
@@ -292,7 +325,9 @@ class FileDnD(wx.Panel):
                 tstamp = '-vf "%s"' % (self.parent.cmdtimestamp)
             else:
                 tstamp = ""
-            IO_tools.stream_play(item, self.parent.time_seq, tstamp)
+            io_tools.stream_play(item, self.parent.time_seq,
+                                 tstamp, self.parent.autoexit
+                                 )
     # ----------------------------------------------------------------------
 
     def delSelect(self, event):
@@ -301,7 +336,8 @@ class FileDnD(wx.Panel):
 
         """
         if not self.selected:
-            self.parent.statusbar_msg(_('No file selected'), 'GOLDENROD')
+            self.parent.statusbar_msg(_('No file selected'), FileDnD.YELLOW,
+                                      FileDnD.BLACK)
         else:
             self.parent.statusbar_msg(_('Add Files'), None)
 
@@ -327,6 +363,8 @@ class FileDnD(wx.Panel):
         self.parent.filedropselected = None
         self.reset_tl()
         self.selected = None
+        self.btn_play.Disable(), self.btn_remove.Disable()
+        self.btn_clear.Disable()
     # ----------------------------------------------------------------------
 
     def on_select(self, event):
@@ -337,6 +375,7 @@ class FileDnD(wx.Panel):
         item = self.flCtrl.GetItemText(index)
         self.parent.filedropselected = item
         self.selected = item
+        self.btn_play.Enable(), self.btn_remove.Enable()
     # ----------------------------------------------------------------------
 
     def on_doubleClick(self, row):
@@ -354,6 +393,7 @@ class FileDnD(wx.Panel):
         """
         self.parent.filedropselected = None
         self.selected = None
+        self.btn_play.Disable(), self.btn_remove.Disable()
     # ----------------------------------------------------------------------
 
     def on_file_save(self, path):
@@ -366,8 +406,9 @@ class FileDnD(wx.Panel):
         self.file_dest = '%s' % (path)
     # -----------------------------------------------------------------------
 
-    def statusbar_msg(self, mess, color):
+    def statusbar_msg(self, mess, bcolor, fcolor=None):
         """
         Set a status bar message of the parent method.
+        bcolor: background, fcolor: foreground
         """
-        self.parent.statusbar_msg('%s' % mess, color)
+        self.parent.statusbar_msg('%s' % mess, bcolor, fcolor)
