@@ -1,35 +1,38 @@
 # -*- coding: UTF-8 -*-
-# Name: youtubedl_ui.py
-# Porpose: youtube-dl user interface
-# Compatibility: Python3, wxPython Phoenix
-# Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
-# Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
-# license: GPL3
-# Rev: Dec.31.2020 *PEP8 compatible*
-#########################################################
+"""
+Name: youtubedl_ui.py
+Porpose: youtube-dl user interface
+Compatibility: Python3, wxPython Phoenix
+Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
+Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
+license: GPL3
+Rev: Aug.14.2021 *-pycodestyle- compatible*
+########################################################
 
-# This file is part of Videomass.
+This file is part of Videomass.
 
-#    Videomass is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation, either version 3 of the License, or
-#    (at your option) any later version.
+   Videomass is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
-#    Videomass is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+   Videomass is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-#    You should have received a copy of the GNU General Public License
-#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
-
-#########################################################
-from videomass3.vdms_io import IO_tools
+   You should have received a copy of the GNU General Public License
+   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
+"""
+import sys
+import wx
+import wx.lib.scrolledpanel as scrolled
+from videomass3.vdms_io import io_tools
 from videomass3.vdms_utils.utils import format_bytes
 from videomass3.vdms_utils.utils import timehuman
 from videomass3.vdms_frames.ydl_mediainfo import YDL_Mediainfo
-import wx
-import wx.lib.scrolledpanel as scrolled
+from videomass3.vdms_utils.get_bmpfromsvg import get_bmp
+from videomass3.vdms_dialogs.playlist_indexing import Indexing
 
 if not hasattr(wx, 'EVT_LIST_ITEM_CHECKED'):
     import wx.lib.mixins.listctrl as listmix
@@ -74,14 +77,13 @@ class Downloader(wx.Panel):
 
     """
     get = wx.GetApp()  # get videomass wx.App attribute
-    OS = get.OS
-    PYLIB_YDL = get.pylibYdl
+    appdata = get.appset
+    icons = get.iconset
 
     MSG_1 = _('At least one "Format Code" must be checked for each '
               'URL selected in green.')
     MSG_2 = _('Function available only if you choose "Download by '
               'format code"')
-
     # light
     LAVENDER = '#e6e6faff'
     # dark
@@ -93,9 +95,25 @@ class Downloader(wx.Panel):
     D_GREEN = '#008000'
     RED = '#ff0000ff'
     GREEN = '#008000'
-    YELLOW = '#bd9f00ff'
+    # YELLOW = '#bd9f00ff'
+    YELLOW = '#bd9f00'
+    WHITE = '#fbf4f4'  # white for background status bar
+    BLACK = '#060505'  # black for background status bar
+    VIOLET = '#D64E93'
 
-    VQUALITY = {('Best quality video'): ['best', 'best'],
+    if appdata['icontheme'] in ('Breeze-Blues',
+                                'Videomass-Colours'):
+        BACKGRD = SOLARIZED
+
+    elif appdata['icontheme'] in ('Breeze-Blues',
+                                  'Breeze-Dark',
+                                  'Videomass-Dark'):
+        BACKGRD = DARK_SLATE
+
+    else:
+        BACKGRD = LAVENDER
+
+    VQUALITY = {('Best quality video'): ['', ''],
                 ('Worst quality video'): ['worst', 'worst']
                 }
     AFORMATS = {
@@ -125,6 +143,17 @@ class Downloader(wx.Panel):
         informations getting by extract_info method from youtube_dl
         module.
         """
+        self.parent = parent
+
+        if 'wx.svg' in sys.modules:  # available only in wx version 4.1 to up
+            bmpplay = get_bmp(Downloader.icons['preview'], ((16, 16)))
+            bmplistindx = get_bmp(Downloader.icons['listindx'], ((16, 16)))
+        else:
+            bmpplay = wx.Bitmap(Downloader.icons['preview'],
+                                wx.BITMAP_TYPE_ANY)
+            bmplistindx = wx.Bitmap(Downloader.icons['listindx'],
+                                    wx.BITMAP_TYPE_ANY)
+
         self.opt = {("NO_PLAYLIST"): [True, "--no-playlist"],
                     ("THUMB"): [False, ""],
                     ("METADATA"): [False, ""],
@@ -133,14 +162,11 @@ class Downloader(wx.Panel):
                     ("A_QUALITY"): ["best", "best"],
                     ("SUBTITLES"): [False, ""],
                     }
-        self.parent = parent
+        self.plidx = {'': ''}
         self.info = list()  # has data information for Show More button
         self.format_dict = dict()  # format codes order with URL matching
         self.oldwx = None  # test result of hasattr EVT_LIST_ITEM_CHECKED
-        '''
-        wx.Panel.__init__(self, parent, -1, style=wx.TAB_TRAVERSAL |
-                          wx.BORDER_THEME)
-        '''
+
         wx.Panel.__init__(self, parent, -1, style=wx.TAB_TRAVERSAL)
         """constructor"""
         sizer_base = wx.BoxSizer(wx.VERTICAL)
@@ -160,7 +186,6 @@ class Downloader(wx.Panel):
                                 )
         self.choice.SetSelection(0)
         boxoptions.Add(self.choice, 0, wx.ALL | wx.EXPAND, 5)
-        f = [x for x in Downloader.VQUALITY.keys()]
         boxoptions.Add((5, 5))
         line0 = wx.StaticLine(self, wx.ID_ANY, pos=wx.DefaultPosition,
                               size=wx.DefaultSize, style=wx.LI_HORIZONTAL,
@@ -168,34 +193,33 @@ class Downloader(wx.Panel):
                               )
         boxoptions.Add(line0, 0, wx.ALL | wx.EXPAND, 5)
         boxoptions.Add((5, 5))
-        panelscroll = scrolled.ScrolledPanel(self, -1, size=(260, 300),
+        panelscroll = scrolled.ScrolledPanel(self, -1, size=(260, 700),
                                              style=wx.TAB_TRAVERSAL |
                                              wx.BORDER_THEME, name="panelscr",
                                              )
         fgs1 = wx.BoxSizer(wx.VERTICAL)
         # fgs1.Add((5, 5))
-        self.cmbx_vq = wx.ComboBox(panelscroll, wx.ID_ANY, choices=f,
+        self.cmbx_vq = wx.ComboBox(panelscroll, wx.ID_ANY,
+                                   choices=list(Downloader.VQUALITY.keys()),
                                    size=(-1, -1), style=wx.CB_DROPDOWN |
                                    wx.CB_READONLY
                                    )
         self.cmbx_vq.SetSelection(0)
         # grid_v.Add((20, 20), 0,)
         fgs1.Add(self.cmbx_vq, 0, wx.ALL | wx.EXPAND, 5)
-        self.cmbx_aq = wx.ComboBox(
-                            panelscroll, wx.ID_ANY,
-                            choices=[x for x in Downloader.AQUALITY.keys()],
-                            size=(-1, -1), style=wx.CB_DROPDOWN |
-                            wx.CB_READONLY
+        self.cmbx_aq = wx.ComboBox(panelscroll, wx.ID_ANY,
+                                   choices=list(Downloader.AQUALITY.keys()),
+                                   size=(-1, -1),
+                                   style=wx.CB_DROPDOWN | wx.CB_READONLY
                                    )
         self.cmbx_aq.SetSelection(0)
         self.cmbx_aq.Disable()
         # grid_v.Add((20, 20), 0,)
         fgs1.Add(self.cmbx_aq, 0, wx.ALL | wx.EXPAND, 5)
-        self.cmbx_af = wx.ComboBox(
-                            panelscroll, wx.ID_ANY,
-                            choices=[x for x in Downloader.AFORMATS.keys()],
-                            size=(-1, -1), style=wx.CB_DROPDOWN |
-                            wx.CB_READONLY
+        self.cmbx_af = wx.ComboBox(panelscroll, wx.ID_ANY,
+                                   choices=list(Downloader.AFORMATS.keys()),
+                                   size=(-1, -1),
+                                   style=wx.CB_DROPDOWN | wx.CB_READONLY
                                    )
         self.cmbx_af.Disable()
         self.cmbx_af.SetSelection(0)
@@ -205,11 +229,25 @@ class Downloader(wx.Panel):
                               size=wx.DefaultSize, style=wx.LI_HORIZONTAL,
                               name=wx.StaticLineNameStr
                               )
-        fgs1.Add(line1, 0, wx.ALL | wx.EXPAND, 5)
+        fgs1.Add(line1, 0, wx.ALL | wx.EXPAND, 10)
+
+        self.btn_play = wx.Button(panelscroll, wx.ID_ANY, _("Preview"))
+        self.btn_play.SetBitmap(bmpplay, wx.LEFT)
+        self.btn_play.Disable()
+        fgs1.Add(self.btn_play, 0, wx.ALL | wx.EXPAND, 5)
+
         self.ckbx_pl = wx.CheckBox(panelscroll, wx.ID_ANY,
-                                   (_('Download all playlist'))
+                                   (_('Download all videos in playlist'))
                                    )
         fgs1.Add(self.ckbx_pl, 0, wx.ALL, 5)
+
+        self.btn_plidx = wx.Button(panelscroll, wx.ID_ANY,
+                                   (_('Playlist Editor'))
+                                   )
+        self.btn_plidx.SetBitmap(bmplistindx, wx.LEFT)
+        fgs1.Add(self.btn_plidx, 0, wx.ALL | wx.EXPAND, 5)
+        self.btn_plidx.Disable()
+
         self.ckbx_thumb = wx.CheckBox(panelscroll, wx.ID_ANY,
                                       (_('Embed thumbnail in audio file'))
                                       )
@@ -222,6 +260,23 @@ class Downloader(wx.Panel):
                                    (_('Write subtitles to video'))
                                    )
         fgs1.Add(self.ckbx_sb, 0, wx.ALL, 5)
+
+        self.ckbx_w = wx.CheckBox(panelscroll, wx.ID_ANY,
+                                  (_('Prevent overwriting files'))
+                                  )
+        fgs1.Add(self.ckbx_w, 0, wx.ALL, 5)
+
+        self.ckbx_id = wx.CheckBox(panelscroll, wx.ID_ANY,
+                                   (_('Include the video ID\n'
+                                      'in the file names'))
+                                   )
+        fgs1.Add(self.ckbx_id, 0, wx.ALL, 5)
+
+        self.ckbx_restrictFN = wx.CheckBox(panelscroll, wx.ID_ANY,
+                                           (_('Restrict file names'))
+                                           )
+        fgs1.Add(self.ckbx_restrictFN, 0, wx.ALL, 5)
+
         boxoptions.Add(panelscroll, 0, wx.ALL | wx.CENTRE, 0)
 
         panelscroll.SetSizer(fgs1)
@@ -244,14 +299,14 @@ class Downloader(wx.Panel):
         # -------------textctrl
         labtstr = _('Help viewer')
         self.labtxt = wx.StaticText(self, label=labtstr)
-        sizer_base.Add(self.labtxt, 0, wx.LEFT, 10)
+        sizer_base.Add(self.labtxt, 0, wx.LEFT, 5)
         self.codText = wx.TextCtrl(self, wx.ID_ANY, "",
                                    style=wx.TE_MULTILINE |
                                    wx.TE_READONLY |
                                    wx.TE_RICH2,
                                    size=(-1, 100)
                                    )
-        sizer_base.Add(self.codText, 0, wx.ALL | wx.EXPAND, 10)
+        sizer_base.Add(self.codText, 0, wx.ALL | wx.EXPAND, 5)
         # -----------------------
         self.codText.Hide(), self.labtxt.Hide()
         self.SetSizer(sizer_base)
@@ -259,32 +314,48 @@ class Downloader(wx.Panel):
         # ----------------------- Properties
         # WARNING do not append text on self.codText here,
         # see `on_Choice meth.`
-        if Downloader.get.THEME == 'Breeze-Blues':
-            self.codText.SetBackgroundColour(Downloader.SOLARIZED)
-        elif Downloader.get.THEME in Downloader.get.DARKicons:
-            self.codText.SetBackgroundColour(Downloader.DARK_SLATE)
-        else:
-            self.codText.SetBackgroundColour(Downloader.LAVENDER)
 
-        if Downloader.OS != 'Darwin':
+        self.codText.SetBackgroundColour(Downloader.BACKGRD)
+
+        if Downloader.appdata['ostype'] != 'Darwin':
             # self.labcode.SetLabelMarkup("<b>%s</b>" % labcstr)
             self.labtxt.SetLabelMarkup("<b>%s</b>" % labtstr)
+        #  tooltips
+        self.btn_play.SetToolTip(_('Play selected url'))
 
         # ----------------------Binder (EVT)----------------------#
         self.choice.Bind(wx.EVT_CHOICE, self.on_Choice)
         self.cmbx_vq.Bind(wx.EVT_COMBOBOX, self.on_Vq)
         self.cmbx_af.Bind(wx.EVT_COMBOBOX, self.on_Af)
         self.cmbx_aq.Bind(wx.EVT_COMBOBOX, self.on_Aq)
+        self.Bind(wx.EVT_BUTTON, self.playSelurl, self.btn_play)
         self.ckbx_pl.Bind(wx.EVT_CHECKBOX, self.on_Playlist)
+        self.btn_plidx.Bind(wx.EVT_BUTTON, self.on_Playlist_idx)
         self.ckbx_thumb.Bind(wx.EVT_CHECKBOX, self.on_Thumbnails)
         self.ckbx_meta.Bind(wx.EVT_CHECKBOX, self.on_Metadata)
         self.ckbx_sb.Bind(wx.EVT_CHECKBOX, self.on_Subtitles)
         self.fcode.Bind(wx.EVT_CONTEXT_MENU, self.onContext)
+        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_select, self.fcode)
+        self.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.on_deselect, self.fcode)
         if self.oldwx is False:
             self.fcode.Bind(wx.EVT_LIST_ITEM_CHECKED, self.onCheck)
             self.fcode.Bind(wx.EVT_LIST_ITEM_UNCHECKED, self.onCheck)
     # -----------------------------------------------------------------#
 
+    def on_select(self, event):
+        """
+        self.fcod selection event that enables btn_play
+        """
+        self.btn_play.Enable()
+    # ----------------------------------------------------------------------
+
+    def on_deselect(self, event):
+        """
+        self.fcod de-selection event that disables btn_play
+        """
+        self.btn_play.Disable()
+    # ----------------------------------------------------------------------
+
     def onCheck(self, event):
         """
         get data from row in the enabled checkbox and set the values
@@ -302,10 +373,10 @@ class Downloader(wx.Panel):
                     self.codText.AppendText('- %s\n' % (Downloader.MSG_2))
             return
 
-        if not self.parent.sb.GetStatusText() == 'Youtube Downloader':
-            self.parent.statusbar_msg('Youtube Downloader', None)
+        if not self.parent.sb.GetStatusText() == 'Ready':
+            self.parent.statusbar_msg('Ready', None)
 
-        if Downloader.PYLIB_YDL is not None:  # YuotubeDL isn't used as module
+        if Downloader.appdata['PYLIBYDL'] is not None:  # isn't used as module
             viddisp, auddisp = 'video ', 'audio '
         else:
             viddisp, auddisp = 'video', 'audio only'
@@ -370,7 +441,37 @@ class Downloader(wx.Panel):
         menuItem = menu.FindItemById(itemId)
 
         if menuItem.GetItemLabel() == _("Play selected url"):
-            self.parent.ExportPlay(self)
+            self.playSelurl(self)
+    # ----------------------------------------------------------------------
+
+    def playSelurl(self, event):
+        """
+        Playback urls
+        """
+        tstamp = '-vf "%s"' % self.parent.cmdtimestamp if \
+                 self.parent.checktimestamp else ''
+
+        if self.fcode.GetSelectedItemCount() == 0:
+            self.parent.statusbar_msg(_('An item must be selected in the '
+                                        'URLs checklist'), Downloader.YELLOW,
+                                      Downloader.BLACK)
+            return
+        else:
+            self.parent.statusbar_msg(_('Ready'), None)
+            item = self.fcode.GetFocusedItem()
+            url = self.fcode.GetItemText(item, 1)
+            if 'playlist' in url or 'channel' in url:
+                # prevent opening too many of ffplay windows
+                wx.MessageBox(_('Videomass cannot play channel or playlists.'),
+                              _('Videomass'), wx.ICON_INFORMATION, self)
+                return
+
+            if self.choice.GetSelection() in [0, 1, 2]:
+                quality = self.fcode.GetItemText(item, 3)
+            elif self.choice.GetSelection() == 3:
+                quality = self.fcode.GetItemText(item, 0)
+
+            io_tools.url_play(url, quality, tstamp, self.parent.autoexit)
     # ----------------------------------------------------------------------
 
     def get_libraryformatcode(self):
@@ -394,10 +495,10 @@ class Downloader(wx.Panel):
         self.fcode.InsertColumn(8, (_('Size')), width=100)
         index = 0
         for link in self.parent.data_url:
-            data = IO_tools.youtubedl_getstatistics(link)
+            data = io_tools.youtubedl_getstatistics(link)
             for meta in data:
                 if meta[1]:
-                    # self.parent.statusbar_msg('Youtube Downloader', None)
+                    # self.parent.statusbar_msg('Ready', None)
                     wx.MessageBox(meta[1], 'Videomass', wx.ICON_ERROR)
                     return True
 
@@ -437,10 +538,10 @@ class Downloader(wx.Panel):
         otherwise return None as exit staus.
         """
         for link in self.parent.data_url:
-            data = IO_tools.youtubedl_getstatistics(link)
+            data = io_tools.youtubedl_getstatistics(link)
             for meta in data:
                 if meta[1]:
-                    # self.parent.statusbar_msg('Youtube Downloader', None)
+                    # self.parent.statusbar_msg('Ready', None)
                     wx.MessageBox(meta[1], 'Videomass', wx.ICON_ERROR)
                     del self.info[:]
                     return True
@@ -488,7 +589,7 @@ class Downloader(wx.Panel):
 
         index = 0
         for link in self.parent.data_url:
-            data = IO_tools.youtube_getformatcode_exec(link)
+            data = io_tools.youtube_getformatcode_exec(link)
             for meta in data:
                 if meta[1]:
                     wx.MessageBox(meta[0], 'Videomass', wx.ICON_ERROR)
@@ -522,14 +623,13 @@ class Downloader(wx.Panel):
         """
         Populate list control with new incoming as urls and
         related resolutions.
+
         """
-        if not self.parent.sb.GetStatusText() == 'Youtube Downloader':
-            self.parent.statusbar_msg('Youtube Downloader', None)
         self.fcode.ClearAll()
         if self.oldwx is False:
             self.fcode.EnableCheckBoxes(enable=False)
         self.fcode.InsertColumn(0, ('#'), width=30)
-        self.fcode.InsertColumn(1, (_('Url')), width=500)
+        self.fcode.InsertColumn(1, (_('Url')), width=400)
         self.fcode.InsertColumn(2, (_('Title')), width=50)
         self.fcode.InsertColumn(3, (_('Quality')), width=250)
 
@@ -547,8 +647,10 @@ class Downloader(wx.Panel):
         """
         show URL data information. This method is called by
         main frame when the 'Show More' button is pressed.
+
         """
-        if Downloader.PYLIB_YDL is not None:  # YuotubeDL is not used as module
+        if Downloader.appdata['PYLIBYDL'] is not None:
+            # YuotubeDL not used as module
             wx.MessageBox(_('Sorry, this feature is disabled.'),
                           'Videomass', wx.ICON_INFORMATION)
             return
@@ -558,7 +660,7 @@ class Downloader(wx.Panel):
             if ret:
                 return
 
-        miniframe = YDL_Mediainfo(self.info, Downloader.OS)
+        miniframe = YDL_Mediainfo(self.info, Downloader.appdata['ostype'])
         miniframe.Show()
     # -----------------------------------------------------------------#
 
@@ -567,7 +669,8 @@ class Downloader(wx.Panel):
         Evaluate which method to call to enable download from "Format Code"
 
         """
-        if Downloader.PYLIB_YDL is not None:  # YuotubeDL isn't used as module
+        if Downloader.appdata['PYLIBYDL'] is not None:
+            # YuotubeDL isn't used as module
             ret = self.get_executableformatcode()
             if ret:
                 return  # do not enable fcode
@@ -579,12 +682,18 @@ class Downloader(wx.Panel):
 
     # -----------------------------------------------------------------#
 
-    def on_Choice(self, event):
+    def on_Choice(self, event, statusmsg=True):
         """
         - Enable or disable some widgets during switching choice box.
         - Set media quality parameter for on_urls_list method
         """
+
+        if statusmsg is True:
+            if not self.parent.sb.GetStatusText() == 'Ready':
+                self.parent.statusbar_msg(_('Ready'), None)
+
         self.codText.Clear()
+        self.btn_play.Disable()
 
         if self.choice.GetSelection() == 0:
             self.cmbx_af.Disable(), self.cmbx_aq.Disable()
@@ -608,7 +717,16 @@ class Downloader(wx.Panel):
             self.on_urls_list('')
 
         elif self.choice.GetSelection() == 3:
-            # self.codText.AppendText(tip3)
+            ceckurls = [url for url in self.parent.data_url if 'playlist'
+                        in url or 'channel' in url]
+            if ceckurls:
+
+                wx.MessageBox(_("Unable to get format codes on playlists "
+                                "and channels"),
+                              "Videomass", wx.ICON_ERROR, self)
+                # prevent KeyError: 'format_id'
+                self.choice.SetSelection(0)
+                return
             self.labtxt.Show(), self.codText.Show()
             self.Layout()
             self.cmbx_vq.Disable(), self.cmbx_aq.Disable()
@@ -668,8 +786,33 @@ class Downloader(wx.Panel):
     def on_Playlist(self, event):
         if self.ckbx_pl.IsChecked():
             self.opt["NO_PLAYLIST"] = [False, "--yes-playlist"]
+            self.btn_plidx.Enable()
         else:
             self.opt["NO_PLAYLIST"] = [True, "--no-playlist"]
+            self.btn_plidx.SetBackgroundColour(wx.NullColour)
+            self.btn_plidx.Disable()
+            self.plidx = {'': ''}
+    # -----------------------------------------------------------------#
+
+    def on_Playlist_idx(self, event):
+        """
+        Set up custom indexing when you download playlists
+        """
+
+        idxdialog = Indexing(self, self.parent.data_url, self.plidx)
+        retcode = idxdialog.ShowModal()
+        if retcode == wx.ID_OK:
+            data = idxdialog.GetValue()
+            if not data:
+                self.btn_plidx.SetBackgroundColour(wx.NullColour)
+                self.plidx = {'': ''}
+            else:
+                self.btn_plidx.SetBackgroundColour(
+                    wx.Colour(Downloader.VIOLET))
+                self.plidx = data
+        else:
+            idxdialog.Destroy()
+            return
     # -----------------------------------------------------------------#
 
     def on_Thumbnails(self, event):
@@ -693,69 +836,88 @@ class Downloader(wx.Panel):
             self.opt["SUBTITLES"] = [False, ""]
     # -----------------------------------------------------------------#
 
+    def getformatcode(self, urls):
+        """
+        Call by on on_Start. Return format code list
+
+        """
+        format_code = []
+
+        for url, key, val in zip(urls,
+                                 self.format_dict.keys(),
+                                 self.format_dict.values()
+                                 ):
+            if key == url:
+                video, audio = '', ''
+                if len(val) == 1:
+                    if val[0].startswith('Audio: '):
+                        audio = val[0].split('Audio: ')[1]
+                    elif val[0].startswith('Video: '):
+                        video = val[0].split('Video: ')[1]
+                    else:
+                        video = val[0].split('Video: ')[1]
+                else:
+                    index_1, index_2 = 0, 0
+                    for i in val:
+                        if i.startswith('Video: '):
+                            index_1 += 1
+                            if index_1 > 1:
+                                video += '/%s' % i.split('Video: ')[1]
+                            else:
+                                video = i.split('Video: ')[1]
+
+                        elif i.startswith('Audio: '):
+                            index_2 += 1
+                            if index_2 > 1:
+                                audio += '/%s' % i.split('Audio: ')[1]
+                            else:
+                                audio = i.split('Audio: ')[1]
+                        else:
+                            index_1 += 1
+                            if index_1 > 1:
+                                video += '/%s' % i.split('Video: ')[1]
+                            else:
+                                video = i.split('Video: ')[1]
+
+                if video and audio:
+                    format_code.append('%s+%s' % (video, audio))
+                elif video:
+                    format_code.append('%s' % video)
+                elif audio:
+                    format_code.append('%s' % audio)
+
+        if len(format_code) != len(urls):
+            return None
+        return format_code
+    # -----------------------------------------------------------------#
+
     def on_start(self):
         """
         Builds command string to use with an embed youtube_dl as
-        python library or using standard youtube-dl command line
-        with subprocess. This depends on some cases.
+        python library or using standard youtube-dl command line.
         """
         urls = self.parent.data_url
 
-        def _getformatcode():
-            """
-            return format code list
-            """
-            format_code = []
+        if not self.ckbx_pl.IsChecked():
+            if [url for url in urls if 'playlist' or 'channel' in url]:
+                if wx.MessageBox(_('The URLs contain playlists or channels. '
+                                   'Are you sure you want to continue?'),
+                                 _('Please confirm'), wx.ICON_QUESTION |
+                                 wx.YES_NO, self) == wx.NO:
+                    return
 
-            for url, key, val in zip(urls,
-                                     self.format_dict.keys(),
-                                     self.format_dict.values()):
-                if key == url:
-                    video, audio = '', ''
-                    if len(val) == 1:
-                        if val[0].startswith('Audio: '):
-                            audio = val[0].split('Audio: ')[1]
-                        elif val[0].startswith('Video: '):
-                            video = val[0].split('Video: ')[1]
-                        else:
-                            video = val[0].split('Video: ')[1]
-                    else:
-                        index_1, index_2 = 0, 0
-                        for i in val:
-                            if i.startswith('Video: '):
-                                index_1 += 1
-                                if index_1 > 1:
-                                    video += '/%s' % i.split('Video: ')[1]
-                                else:
-                                    video = i.split('Video: ')[1]
+        if self.ckbx_id.IsChecked():
+            _id = '%(title).100s-%(id)s'
+        else:
+            _id = '%(title).100s'
 
-                            elif i.startswith('Audio: '):
-                                index_2 += 1
-                                if index_2 > 1:
-                                    audio += '/%s' % i.split('Audio: ')[1]
-                                else:
-                                    audio = i.split('Audio: ')[1]
-                            else:
-                                index_1 += 1
-                                if index_1 > 1:
-                                    video += '/%s' % i.split('Video: ')[1]
-                                else:
-                                    video = i.split('Video: ')[1]
+        if Downloader.appdata['PYLIBYDL'] is None:  # is used as library
 
-                    if video and audio:
-                        format_code.append('%s+%s' % (video, audio))
-                    elif video:
-                        format_code.append('%s' % video)
-                    elif audio:
-                        format_code.append('%s' % audio)
-
-            if len(format_code) != len(urls):
-                return None
-            return format_code
-
-        if Downloader.PYLIB_YDL is None:  # youtube-dl is used as library
             logname = 'youtubedl_lib.log'
+            nooverwrites = True if self.ckbx_w.IsChecked() else False
+            restrictfn = True if self.ckbx_restrictFN.IsChecked() else False
             postprocessors = []
+
             if self.choice.GetSelection() == 2:
                 postprocessors.append({'key': 'FFmpegExtractAudio',
                                        'preferredcodec':
@@ -775,57 +937,70 @@ class Downloader(wx.Panel):
                 code = []
                 data = {'format': self.opt["V_QUALITY"][0],
                         'noplaylist': self.opt["NO_PLAYLIST"][0],
+                        'playlist_items': self.plidx,
+                        'nooverwrites': nooverwrites,
                         'writethumbnail': self.opt["THUMB"][0],
-                        'outtmpl': '%(title)s.%(ext)s',
+                        'outtmpl': '{0}.%(ext)s'.format(_id),
                         'extractaudio': False,
                         'addmetadata': self.opt["METADATA"][0],
                         'writesubtitles': self.opt["SUBTITLES"][0],
                         'writeautomaticsub': self.opt["SUBTITLES"][0],
                         'allsubtitles': self.opt["SUBTITLES"][0],
-                        'postprocessors': postprocessors
+                        'postprocessors': postprocessors,
+                        'restrictfilenames': restrictfn,
                         }
             if self.choice.GetSelection() == 1:  # audio files and video files
                 code = []
                 data = {'format': '%svideo,%saudio' %
                         (self.opt["V_QUALITY"][0], self.opt["A_QUALITY"][0]),
                         'noplaylist': self.opt["NO_PLAYLIST"][0],
+                        'playlist_items': self.plidx,
+                        'nooverwrites': nooverwrites,
                         'writethumbnail': self.opt["THUMB"][0],
-                        'outtmpl': '%(title)s.f%(format_id)s.%(ext)s',
+                        'outtmpl': '{0}.f%(format_id)s.%(ext)s'.format(_id),
                         'extractaudio': False,
                         'addmetadata': self.opt["METADATA"][0],
                         'writesubtitles': self.opt["SUBTITLES"][0],
                         'writeautomaticsub': self.opt["SUBTITLES"][0],
                         'allsubtitles': self.opt["SUBTITLES"][0],
-                        'postprocessors': postprocessors
+                        'postprocessors': postprocessors,
+                        'restrictfilenames': restrictfn,
                         }
             elif self.choice.GetSelection() == 2:  # audio only
                 code = []
                 data = {'format': 'best',
                         'noplaylist': self.opt["NO_PLAYLIST"][0],
+                        'playlist_items': self.plidx,
+                        'nooverwrites': nooverwrites,
                         'writethumbnail': self.opt["THUMB"][0],
-                        'outtmpl': '%(title)s.%(ext)s',
+                        'outtmpl': '{0}.%(ext)s'.format(_id),
                         'extractaudio': True,
                         'addmetadata': self.opt["METADATA"][0],
                         'writesubtitles': self.opt["SUBTITLES"][0],
                         'writeautomaticsub': self.opt["SUBTITLES"][0],
                         'allsubtitles': self.opt["SUBTITLES"][0],
-                        'postprocessors': postprocessors
+                        'postprocessors': postprocessors,
+                        'restrictfilenames': restrictfn,
                         }
             if self.choice.GetSelection() == 3:  # format code
-                code = _getformatcode()
+                code = self.getformatcode(urls)
                 if not code:
-                    self.parent.statusbar_msg(Downloader.MSG_1, Downloader.RED)
+                    self.parent.statusbar_msg(Downloader.MSG_1, Downloader.RED,
+                                              Downloader.WHITE)
                     return
                 data = {'format': '',
                         'noplaylist': self.opt["NO_PLAYLIST"][0],
+                        'playlist_items': self.plidx,
+                        'nooverwrites': nooverwrites,
                         'writethumbnail': self.opt["THUMB"][0],
-                        'outtmpl': '%(title)s.f%(format_id)s.%(ext)s',
+                        'outtmpl': '{0}.f%(format_id)s.%(ext)s'.format(_id),
                         'extractaudio': False,
                         'addmetadata': self.opt["METADATA"][0],
                         'writesubtitles': self.opt["SUBTITLES"][0],
                         'writeautomaticsub': self.opt["SUBTITLES"][0],
                         'allsubtitles': self.opt["SUBTITLES"][0],
-                        'postprocessors': postprocessors
+                        'postprocessors': postprocessors,
+                        'restrictfilenames': restrictfn,
                         }
             self.parent.switch_to_processing('youtube_dl python package',
                                              urls,
@@ -841,6 +1016,11 @@ class Downloader(wx.Panel):
         else:  # ----------- with youtube-dl command line execution
 
             logname = 'youtubedl_exec.log'
+            nooverwrites = '--no-overwrites' if self.ckbx_w.IsChecked() else ''
+            if self.ckbx_restrictFN.IsChecked():
+                restrictfn = '--restrict-filenames'
+            else:
+                restrictfn = ''
 
             if self.choice.GetSelection() == 0:  # default
                 code = []
@@ -850,7 +1030,10 @@ class Downloader(wx.Panel):
                         f'{self.opt["SUBTITLES"][1]} '
                         f'{self.opt["THUMB"][1]} '
                         f'{self.opt["NO_PLAYLIST"][1]}'),
-                       ('%(title)s.%(ext)s')
+                       ('{0}.%(ext)s'.format(_id)),
+                       f'{nooverwrites}',
+                       f'{restrictfn}',
+                       self.plidx,
                        ]
 
             if self.choice.GetSelection() == 1:  # audio files + video files
@@ -862,8 +1045,12 @@ class Downloader(wx.Panel):
                         f'{self.opt["SUBTITLES"][1]} '
                         f'{self.opt["THUMB"][1]} '
                         f'{self.opt["NO_PLAYLIST"][1]}'),
-                       ('%(title)s.f%(format_id)s.%(ext)s')
+                       ('{0}.f%(format_id)s.%(ext)s'.format(_id)),
+                       f'{nooverwrites}',
+                       f'{restrictfn}',
+                       self.plidx,
                        ]
+
             elif self.choice.GetSelection() == 2:  # audio only
                 code = []
                 cmd = [(f'{self.opt["A_FORMAT"][1]} '
@@ -871,18 +1058,26 @@ class Downloader(wx.Panel):
                         f'{self.opt["SUBTITLES"][1]} '
                         f'{self.opt["THUMB"][1]} '
                         f'{self.opt["NO_PLAYLIST"][1]}'),
-                       ('%(title)s.%(ext)s')
+                       ('{0}.%(ext)s'.format(_id)),
+                       f'{nooverwrites}',
+                       f'{restrictfn}',
+                       self.plidx,
                        ]
+
             if self.choice.GetSelection() == 3:  # format code
-                code = _getformatcode()
+                code = self.getformatcode(urls)
                 if not code:
-                    self.parent.statusbar_msg(Downloader.MSG_1, Downloader.RED)
+                    self.parent.statusbar_msg(Downloader.MSG_1, Downloader.RED,
+                                              Downloader.WHITE)
                     return
                 cmd = [(f'{self.opt["METADATA"][1]} '
                         f'{self.opt["SUBTITLES"][1]} '
                         f'{self.opt["THUMB"][1]} '
                         f'{self.opt["NO_PLAYLIST"][1]}'),
-                       ('%(title)s.f%(format_id)s.%(ext)s')
+                       ('{0}.f%(format_id)s.%(ext)s'.format(_id)),
+                       f'{nooverwrites}',
+                       f'{restrictfn}',
+                       self.plidx,
                        ]
             self.parent.switch_to_processing('youtube-dl executable',
                                              urls,
