@@ -1,28 +1,31 @@
 # -*- coding: UTF-8 -*-
-# Name: youtubedlupdater.py
-# Porpose: update tasks
-# Compatibility: Python3, wxPython4 Phoenix
-# Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
-# Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
-# license: GPL3
-# Rev: Oct.03.2020 *PEP8 compatible*
-#########################################################
-# This file is part of Videomass.
+"""
+Name: youtubedlupdater.py
+Porpose: update tasks
+Compatibility: Python3, wxPython4 Phoenix
+Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
+Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
+license: GPL3
+Rev: Aug.02.2021
+Code checker:
+    flake8: --ignore F821, W504
+    pylint: --ignore E0602, E1101
 
-#    Videomass is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation, either version 3 of the License, or
-#    (at your option) any later version.
+This file is part of Videomass.
 
-#    Videomass is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+   Videomass is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
-#    You should have received a copy of the GNU General Public License
-#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
+   Videomass is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-#########################################################
+   You should have received a copy of the GNU General Public License
+   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
+"""
 import subprocess
 import shlex
 import platform
@@ -31,9 +34,10 @@ import os
 import shutil
 import ssl
 import urllib.request
+from threading import Thread
 import wx
 from pubsub import pub
-from threading import Thread
+
 
 '''
 class CheckNewRelease(Thread):
@@ -79,13 +83,14 @@ class CheckNewRelease(Thread):
 # -------------------------------------------------------------------------#
 
 
-class Command_Execution(Thread):
+class CmdExec(Thread):
     """
     Executes generic command line with an executable, e.g.
     - read the installed version of youtube-dl
     - update the downloaded sources of youtube-dl
 
     """
+
     def __init__(self, cmd):
         """
         OS: Operative System id
@@ -93,14 +98,12 @@ class Command_Execution(Thread):
         self.status: tuple object with exit status of the process
         self.data: returned output of the self.status
         """
-        self.OS = platform.system()
+        self.ostype = platform.system()
         self.cmd = cmd
         self.data = None
         self.status = None
 
         Thread.__init__(self)
-        """initialize"""
-
         self.start()  # start the thread (va in self.run())
     # ----------------------------------------------------------------#
 
@@ -109,7 +112,7 @@ class Command_Execution(Thread):
         Execute command line via subprocess class and get output
         at the end of the process.
         """
-        if self.OS == 'Windows':
+        if self.ostype == 'Windows':
             cmd = " ".join(self.cmd)
             info = subprocess.STARTUPINFO()
             info.dwFlags |= subprocess.STARTF_USESHOWWINDOW
@@ -119,27 +122,29 @@ class Command_Execution(Thread):
             info = None
 
         try:
-            p = subprocess.Popen(cmd,
-                                 stdout=subprocess.PIPE,
-                                 stderr=subprocess.STDOUT,
-                                 universal_newlines=True,  # mod text
-                                 startupinfo=info,
-                                 )
-            out = p.communicate()
+            with subprocess.Popen(cmd,
+                                  stdout=subprocess.PIPE,
+                                  stderr=subprocess.STDOUT,
+                                  universal_newlines=True,  # mod text
+                                  startupinfo=info,
+                                  ) as proc:
+
+                out = proc.communicate()
+
+                if proc.returncode:  # if returncode == 1
+                    if not out[0] and not out[1] and self.ostype == 'Windows':
+                        self.status = (_('Requires MSVCR100.dll\nTo resolve '
+                                         'this problem install: Microsoft '
+                                         'Visual C++ 2010 Redistributable '
+                                         'Package (x86)'), 'error')
+                    else:
+                        self.status = (out[0], 'error')
+                else:
+                    self.status = (out[0], out[1])
 
         except (OSError, FileNotFoundError) as oserr:  # exec. do not exist
             self.status = ('%s' % oserr, 'error')
-        else:
-            if p.returncode:  # if returncode == 1
-                if not out[0] and not out[1] and self.OS == 'Windows':
-                    self.status = (_('Requires MSVCR100.dll\nTo resolve this '
-                                     'problem install: Microsoft Visual C++ '
-                                     '2010 Redistributable Package (x86)',
-                                     'error'))
-                else:
-                    self.status = (out[0], 'error')
-            else:
-                self.status = (out[0], out[1])
+
         self.data = self.status
 
         wx.CallAfter(pub.sendMessage,
@@ -149,10 +154,12 @@ class Command_Execution(Thread):
 # ---------------------------------------------------------------------#
 
 
-class Upgrade_Latest(Thread):
+'''
+class UpgradeLatest(Thread):
     """
     Download latest version of youtube-dl.exe, see self.url .
     """
+
     def __init__(self, url, dest):
         """
         Attributes defined here:
@@ -168,8 +175,6 @@ class Upgrade_Latest(Thread):
         self.status = None
 
         Thread.__init__(self)
-        """initialize"""
-
         self.start()  # start the thread (va in self.run())
     # ----------------------------------------------------------------#
 
@@ -183,7 +188,7 @@ class Upgrade_Latest(Thread):
         page = urllib.request.Request(self.url, headers=headers)
         try:
             with urllib.request.urlopen(page, context=context) as \
-                 response, open(self.dest, 'wb') as out_file:
+                    response, open(self.dest, 'wb') as out_file:
                 shutil.copyfileobj(response, out_file)
 
             self.status = self.url, None
@@ -200,16 +205,18 @@ class Upgrade_Latest(Thread):
                      "RESULT_EVT",
                      status=''
                      )
+'''
 # ---------------------------------------------------------------------#
 
 
-class Update_Youtube_dl_Appimage(Thread):
+class UpdateYoutubedlAppimage(Thread):
     """
     Install or Update `youtube_dl` python package (library) using
     xterm terminal emulator with subprocess for displaying and
     redirecting the output to log file.
 
     """
+
     def __init__(self, log, appimage):
         """
         Attributes defined here:
@@ -236,14 +243,13 @@ class Update_Youtube_dl_Appimage(Thread):
         exe = os.path.join(binpath + '/youtube_dl_update_appimage.sh')
         self.status = None
         self.cmd = shlex.split(
-                        "xterm +hold -u8 -bg 'grey15' -fa 'Monospace' "
-                        "-fs 9 -geometry 120x35 -title '..Updating "
-                        "youtube_dl package on %s' -e '%s %s "
-                        "2>&1 | tee %s'" % (name, exe, appimage, log)
-                        )
-        Thread.__init__(self)
-        """initialize"""
+            "xterm +hold -u8 -bg 'grey15' -fa 'Monospace' "
+            "-fs 9 -geometry 120x35 -title '..Updating "
+            "youtube_dl package on %s' -e '%s %s "
+            "2>&1 | tee %s'" % (name, exe, appimage, log)
+        )
 
+        Thread.__init__(self)
         self.start()  # start the thread
     # ----------------------------------------------------------------#
 
@@ -253,14 +259,14 @@ class Update_Youtube_dl_Appimage(Thread):
 
         """
         try:
-            p = subprocess.run(self.cmd, shell=False)
+            proc = subprocess.run(self.cmd, shell=False)
 
         except FileNotFoundError as err:
             self.status = err
 
-        if p.returncode:
-            self.status = "EXIT: %s\nERROR: %s" % (p.returncode, p.stderr)
-
+        if proc.returncode:
+            self.status = "EXIT: %s\nERROR: %s" % (proc.returncode,
+                                                   proc.stderr)
         wx.CallAfter(pub.sendMessage,
                      "RESULT_EVT",
                      status=''
